==> mupad-1.4.2.tgz: mupad/share/mmg/include/kernel/MSV_save.h <==
/* $Date: 1996/11/22 15:48:18 $  $Author: kg $  $Revision: 1.10 $ */

/******************************************************************************/
/* DATEI:       MSV_save.h                                                    */
/* AUTOR:       Holger Naundorf                                               */
/* ERSTELLT:    ??.??.??                                                      */
/* GEAENDERT:   16.01.96                                                      */
/* INHALT:      Definitionen zur Verwaltung des M-Kodes                       */
/*                                                                            */
/* In diesem File werden die Typen und die Makros definiert, die zum Erzeugen */
/* von .m-Code erforderlich sind.                                             */
/*                                                                            */
/******************************************************************************/

#ifndef __MSV_save__
#define __MSV_save__


/*******************************************************************************
** Im folgenden werden die ben"otigten Typen definiert.                       **
*******************************************************************************/
typedef struct
{
   char  *start;                /* Anfang des Speicherbereiches               */
   char  *pos;                  /* Aktuelle Position, an die geschrieben wird */
   long   used_bytes;           /* Anzahl der benutzten Bytes                 */
   long   length;               /* Laenge des verf"ugbaren Speicherbereiches  */
} MSVTfile;

typedef MCLT_bytestream *MSVThandle;
typedef MSVThandle      MSVT_handle;

typedef struct
{
   long  size;
   long  counter;
   long  type;
   long  memtype;
} MSVTheaderinfo;

typedef struct
{
   long  	handle_id;
   long         reference;
   long         free;
   char         complete;       /* Hat dieser Eintrag den Wert '0', so werden */
                                /* Daten des Typs MMM_STRING ohne ihre Soehne */
                                /* abgespeichert, sonst mit.                  */
} MSVT_handlelist;

typedef long MSVT_kind;


/*******************************************************************************
** Im folgenden werden einige Konstanten definiert.                           **
*******************************************************************************/
#define MSVC_bits_of_char    8       /* Anzahl der Bits eines char            */
#define MSVC_long_size       4       /* Aus wievielen Bytes besteht ein long  */
#define MSVC_short_size      2       /* Aus wievielen Bytes besteht ein short */

#define MSVC_S_Pointer       4       /* Dies repraesentiert keinen normalen   */
                                     /* Typen und kann sich deshalb mit den   */
                                     /* normalen Typen ueberschneiden         */

#define MSVC_reference       3       /* Dies repraesentiert keinen normalen   */
                                     /* Typen und kann sich deshalb mit den   */
                                     /* normalen Typen ueberschneiden         */
                                  
#define MSVC_MMMNULL         2       /* Kann eigentlich durch den Wert von    */
                                     /* MSVC_MMMNULL_type ersetzt werden      */
                                  
#define MSVC_DOMAIN_type     1       /* Ein Domain wird hiermit eingeleitet   */
#define MSVC_MMMNULL_type    0       /* MMMNULL wird hiermit eingeleitet      */
#define MSVC_IDENT_type      5
#define MSVC_BOOL_type       6       /* Ein Bool wird hiermit eingeleitet     */

#define MSVC_char            0xff
#define MSVC_upper_bit       0x80
#define MSVC_upper_bit_long  0x80000000
#define MSVC_lower_bits      0x7f
#define MSVC_typ             4L      /* Typ des S-Zeigers, in dem Referenzen  */
                                     /* gespeichert werden                    */
                                  
#define MSVC_bytes_of_short_reference  2L
#define MSVC_short_reference_bit       0x40
#define MSVC_max_short_reference       0x3fff
#define MSVC_appended_put              1
#define MSVC_normal_put                2


/* Zum Ausblenden der Bits, die NICHT dafuer zustaendig sind, zu  */
/* bestimmen, ob gelesen oder geschrieben werden soll.            */

#define MSVC_iomode          3L 

#define MSVC_read            1L
#define MSVC_write           2L
#define MSVC_append          3L

/* Ist dieses Bit beim Oeffnen eines Ausgabekanals gesetzt, so werden */
/* die Daten vom Typ MMM_STRING mit ihren Soehnen rausgeschrieben     */

#define MSVC_complete  4L 


/*************************************************************************
** void MSV_put_number(T zahl , MSVThandle h)                           **
** Der dieser Funktion "ubergebene Parameter zahl kann jeden Typ T      **
** haben auf dem mit >> operiert werden kann und aus dem mittels dieser **
** Operation der gesamte Inhalt ausgelesen werden kann.                 **
*************************************************************************/
#define MSV_put_number(zahl,h)                      \
        { long   i;                                 \
                                                    \
          for( i=sizeof(zahl) ; i>0L ; i-- ) {      \
               MSV_put_char( (char) zahl, h );      \
               zahl >>= MSVC_bits_of_char;          \
          }                                         \
        }


/*************************************************************************
** void MSV_put_Mem(S_Pointer s, MSVThandle h, MSVTheaderinfo *in)      **
** Die Funktion ruft die dem Typ von s entsprechende Funktion zum       **
** Schreiben auf.                                                       **
*************************************************************************/
#define MSV_put_Mem(s,h,in) \
        MSVV_put_Mem_table[(in)->type](s,h,in)


/************************************************************************* 
** void MSV_get_Mem(S_Pointer s, MSVThandle h, MSVTheaderinfo *in)      ** 
** Die Funktion ruft die dem Typ von s entsprechende Funktion zum       ** 
** Lesen auf.                                                           ** 
*************************************************************************/ 
#define MSV_get_Mem(s,h,in) \
        MSVV_get_Mem_table[(in)->type](s,h,in) 
 

/*******************************************************************************
** Prototypen aller Funktionen, die fuer den .m-Code verwendet werden.        **
*******************************************************************************/

void MSV_init            ( VOID );
void MSV_exit            ( VOID );

void MSV_open            ( MSVT_handle h, MSVT_kind  k   );
void MSV_close           ( MSVT_handle h );
long MSV_read            ( MSVT_handle h, S_Pointer *p   );
void MSV_write           ( MSVT_handle h, S_Pointer  val );

void MSV_put_char        ( char c, MSVThandle h );
void MSV_put_type        ( long t, MSVThandle h );
void MSV_put_long        ( long l, MSVThandle h );
void MSV_put_long1       ( long l, MSVThandle h );
void MSV_put_header      ( S_Pointer s, MSVThandle h, MSVTheaderinfo *in );
void MSV_put_Mem_generic ( S_Pointer s, MSVThandle h, MSVTheaderinfo *in );
void MSV_put_Point       ( S_Pointer s, MSVThandle h, MSVTheaderinfo *in );
void MSV_put_S_Pointer   ( S_Pointer s, MSVThandle h );

void MSV_get_char        ( unsigned char *c,    MSVThandle h );
void MSV_get_long        ( long          *l,    MSVThandle h );
void MSV_get_long1       ( long          *l,    MSVThandle h );
int  MSV_get_type        ( long          *type, MSVThandle h );
int  MSV_get_header      ( MSVThandle h, MSVTheaderinfo *in );
void MSV_get_Mem_generic ( S_Pointer  s, MSVThandle h, MSVTheaderinfo *in );
void MSV_get_Point       ( S_Pointer  s, MSVThandle h );
void MSV_get_S_Pointer   ( S_Pointer *s, MSVThandle h );


#endif /* __MSV_save__ */

==> mupad-2.0.0.tgz: mupad/share/mmg/include/kernel/MSV_save.h <==
/* $Date: 2000/03/30 14:25:27 $  $Author: kg $  $Revision: 1.11 $ */

/******************************************************************************/
/* DATEI:       MSV_save.h                                                    */
/* AUTOR:       Holger Naundorf                                               */
/* ERSTELLT:    ??.??.??                                                      */
/* GEAENDERT:   16.01.96                                                      */
/* INHALT:      Definitionen zur Verwaltung des M-Kodes                       */
/*                                                                            */
/* In diesem File werden die Typen und die Makros definiert, die zum Erzeugen */
/* von .m-Code erforderlich sind.                                             */
/*                                                                            */
/******************************************************************************/

#ifndef __MSV_save__
#define __MSV_save__

#include "MCT_iostream.h"

/*******************************************************************************
** Im folgenden werden die ben"otigten Typen definiert.                       **
*******************************************************************************/
typedef struct
{
   char  *start;                /* Anfang des Speicherbereiches               */
   char  *pos;                  /* Aktuelle Position, an die geschrieben wird */
   long   used_bytes;           /* Anzahl der benutzten Bytes                 */
   long   length;               /* Laenge des verf"ugbaren Speicherbereiches  */
} MSVTfile;

typedef MCLT_bytestream *MSVThandle;
typedef MSVThandle      MSVT_handle;

typedef struct
{
   long  size;
   long  counter;
   long  type;
} MSVTheaderinfo;

typedef long MSVT_kind;

typedef struct
{
   long  	handle_id;
   long         reference;
   bool         free;
   MSVT_kind    complete;       /* Hat dieser Eintrag den Wert '0', so werden */
                                /* Daten des Typs MMM_STRING ohne ihre Soehne */
                                /* abgespeichert, sonst mit.                  */
   // Ist das oberste Bit (MSVC_oldmcode_bit) von complete gesetzt, so
   // haben wir alten MCode vorliegen. => Mehrdeutigkeit von complete
} MSVT_handlelist;


/*******************************************************************************
** Im folgenden werden einige Konstanten definiert.                           **
*******************************************************************************/
#define MSVC_bits_of_char    8       /* Anzahl der Bits eines char            */
#define MSVC_long_size       4       /* Aus wievielen Bytes besteht ein long  */
#define MSVC_short_size      2       /* Aus wievielen Bytes besteht ein short */

#define MSVC_S_Pointer       4       /* Dies repraesentiert keinen normalen   */
                                     /* Typen und kann sich deshalb mit den   */
                                     /* normalen Typen ueberschneiden         */

#define MSVC_reference       3       /* Dies repraesentiert keinen normalen   */
                                     /* Typen und kann sich deshalb mit den   */
                                     /* normalen Typen ueberschneiden         */

#define MSVC_MMMNULL         2       /* Kann eigentlich durch den Wert von    */
                                     /* MSVC_MMMNULL_type ersetzt werden      */

#define MSVC_DOMAIN_type     1       /* Ein Domain wird hiermit eingeleitet   */
#define MSVC_MMMNULL_type    0       /* MMMNULL wird hiermit eingeleitet      */
#define MSVC_IDENT_type      5

#define MSVC_char            0xff
#define MSVC_upper_bit       0x80
#define MSVC_upper_bit_long  0x80000000
#define MSVC_lower_bits      0x7f
#define MSVC_typ             4      /* Typ des S-Zeigers, in dem Referenzen  */
                                     /* gespeichert werden                    */

#define MSVC_bytes_of_short_reference  2
#define MSVC_short_reference_bit       0x40
#define MSVC_max_short_reference       0x3fff
#define MSVC_appended_put              1
#define MSVC_normal_put                2

#define MSVC_oldmcode_bit    0x80  // Wird zum markieren in der
                                   // MSVT_handlelist fuer alten Mcode benutzt

/* Zum Ausblenden der Bits, die NICHT dafuer zustaendig sind, zu  */
/* bestimmen, ob gelesen oder geschrieben werden soll.            */

#define MSVC_iomode          3

#define MSVC_read            1
#define MSVC_write           2
#define MSVC_append          3

/* Ist dieses Bit beim Oeffnen eines Ausgabekanals gesetzt, so werden */
/* die Daten vom Typ MMM_STRING mit ihren Soehnen rausgeschrieben     */

#define MSVC_complete  4


/*************************************************************************
** void MSV_put_number(T zahl , MSVThandle h)                           **
** Der dieser Funktion "ubergebene Parameter zahl kann jeden Typ T      **
** haben auf dem mit >> operiert werden kann und aus dem mittels dieser **
** Operation der gesamte Inhalt ausgelesen werden kann.                 **
*************************************************************************/
#define MSV_put_number(zahl,h)                      \
        { long   i;                                 \
                                                    \
          for( i=sizeof(zahl) ; i>0 ; i-- ) {      \
               MSV_put_char( (char) zahl, h );      \
               zahl >>= MSVC_bits_of_char;          \
          }                                         \
        }


/*************************************************************************
** void MSV_put_Mem(S_Pointer s, MSVThandle h, MSVTheaderinfo *in)      **
** Die Funktion ruft die dem Typ von s entsprechende Funktion zum       **
** Schreiben auf.                                                       **
*************************************************************************/
#define MSV_put_Mem(s,h,in) \
        MSVV_put_Mem_table[(in)->type](s,h,in)


/*************************************************************************
** void MSV_get_Mem(S_Pointer s, MSVThandle h, MSVTheaderinfo *in)      **
** Die Funktion ruft die dem Typ von s entsprechende Funktion zum       **
** Lesen auf.                                                           **
*************************************************************************/
#define MSV_get_Mem(s,h,in) \
        MSVV_get_Mem_table[(in)->type](s,h,in)


/*******************************************************************************
** Prototypen aller Funktionen, die fuer den .m-Code verwendet werden.        **
*******************************************************************************/

void MSV_init            (  );
void MSV_exit            (  );

void MSV_open            ( MSVT_handle h, MSVT_kind  k   );
void MSV_close           ( MSVT_handle h );
long MSV_read            ( MSVT_handle h, S_Pointer *p   );
void MSV_write           ( MSVT_handle h, S_Pointer  val, bool flat=false );

void MSV_put_char        ( char c, MSVThandle h );
void MSV_put_type        ( long t, MSVThandle h );
void MSV_put_long        ( long l, MSVThandle h );
void MSV_put_long1       ( long l, MSVThandle h );
void MSV_put_header      ( S_Pointer s, MSVThandle h, MSVTheaderinfo *in );
void MSV_put_Mem_generic ( S_Pointer s, MSVThandle h, MSVTheaderinfo *in );
void MSV_put_Point       ( S_Pointer s, MSVThandle h, MSVTheaderinfo *in );
void MSV_put_S_Pointer   ( S_Pointer s, MSVThandle h );

void MSV_get_char        ( unsigned char *c,    MSVThandle h );
void MSV_get_long        ( long          *l,    MSVThandle h );
void MSV_get_long1       ( long          *l,    MSVThandle h );
int  MSV_get_type        ( long          *type, MSVThandle h );
int  MSV_get_header      ( MSVThandle h, MSVTheaderinfo *in );
void MSV_get_Mem_generic ( S_Pointer  s, MSVThandle h, MSVTheaderinfo *in );
void MSV_get_Point       ( S_Pointer  s, MSVThandle h );
void MSV_get_S_Pointer   ( S_Pointer *s, MSVThandle h );

#endif /* __MSV_save__ */
